@@ -61,9 +61,10 @@ class AdvancedHTMLParser(HTMLParser):
         # Do not automatically convert charrefs in python3
         self.convert_charrefs = False
 
+        # Encoding to use for this document
         self.encoding = encoding
 
-        self.inTag = []
+        self._inTag = []
         self.root = None
         self.doctype = None
 
@@ -100,6 +101,7 @@ class AdvancedHTMLParser(HTMLParser):
             Internal for parsing
         '''
         tagName = tagName.lower()
+        inTag = self._inTag
 
         if isSelfClosing is False and tagName in IMPLICIT_SELF_CLOSING_TAGS:
             isSelfClosing = True
@@ -107,13 +109,13 @@ class AdvancedHTMLParser(HTMLParser):
         newTag = AdvancedTag(tagName, attributeList, isSelfClosing, ownerDocument=self)
         if self.root is None:
             self.root = newTag
-        elif len(self.inTag) > 0:
-            self.inTag[-1].appendChild(newTag)
+        elif len(inTag) > 0:
+            inTag[-1].appendChild(newTag)
         else:
             raise MultipleRootNodeException()
 
         if isSelfClosing is False:
-            self.inTag.append(newTag)
+            inTag.append(newTag)
 
         return newTag
 
@@ -129,18 +131,19 @@ class AdvancedHTMLParser(HTMLParser):
         '''
         try:
             foundIt = False
-            for i in range(len(self.inTag)):
-                if self.inTag[i].tagName == tagName:
+            inTag = self._inTag
+            for i in range(len(inTag)):
+                if inTag[i].tagName == tagName:
                     foundIt = True
                     break
 
             if not foundIt:
                 return
             # Handle closing tags which should have been closed but weren't
-            while self.inTag[-1].tagName != tagName:
-                self.inTag.pop()
+            while inTag[-1].tagName != tagName:
+                inTag.pop()
 
-            self.inTag.pop()
+            inTag.pop()
         except:
             pass
 
@@ -150,8 +153,9 @@ class AdvancedHTMLParser(HTMLParser):
             Internal for parsing
         '''
         if data:
-            if len(self.inTag) > 0:
-                self.inTag[-1].appendText(data)
+            inTag = self._inTag
+            if len(inTag) > 0:
+                inTag[-1].appendText(data)
             elif data.strip(): #and not self.getRoot():
                 # Must be text prior to or after root node
                 raise MultipleRootNodeException()
@@ -160,8 +164,9 @@ class AdvancedHTMLParser(HTMLParser):
         '''
             Internal for parsing
         '''
-        if len(self.inTag) > 0:
-            self.inTag[-1].appendText('&%s;' %(entity,))
+        inTag = self._inTag
+        if len(inTag) > 0:
+            inTag[-1].appendText('&%s;' %(entity,))
         else:
             raise MultipleRootNodeException()
 
@@ -169,8 +174,9 @@ class AdvancedHTMLParser(HTMLParser):
         '''
             Internal for parsing
         '''
-        if len(self.inTag) > 0:
-            self.inTag[-1].appendText('&#%s;' %(charRef,))
+        inTag = self._inTag
+        if len(inTag) > 0:
+            inTag[-1].appendText('&#%s;' %(charRef,))
         else:
             raise MultipleRootNodeException()
 
@@ -178,8 +184,9 @@ class AdvancedHTMLParser(HTMLParser):
         '''
             Internal for parsing
         '''
-        if len(self.inTag) > 0:
-            self.inTag[-1].appendText('<!-- %s -->' %(comment,))
+        inTag = self._inTag
+        if len(inTag) > 0:
+            inTag[-1].appendText('<!-- %s -->' %(comment,))
         else:
             raise MultipleRootNodeException()
 
@@ -265,10 +272,14 @@ class AdvancedHTMLParser(HTMLParser):
         if isFromRoot is True and root.tagName == tagName:
             elements.append(root)
 
+        getElementsByTagName = self.getElementsByTagName
         for child in root.children:
+
             if child.tagName == tagName:
                 elements.append(child)
-            elements += self.getElementsByTagName(tagName, child)
+
+            elements += getElementsByTagName(tagName, child)
+
         return TagCollection(elements)
 
     def getElementsByName(self, name, root='root'):
@@ -285,10 +296,13 @@ class AdvancedHTMLParser(HTMLParser):
         if isFromRoot is True and root.name == name:
             elements.append(root)
 
+        getElementsByName = self.getElementsByName
         for child in root.children:
+
             if child.getAttribute('name') == name:
                 elements.append(child)
-            elements += self.getElementsByName(name, child)
+
+            elements += getElementsByName(name, child)
 
         return TagCollection(elements)
 
@@ -304,12 +318,16 @@ class AdvancedHTMLParser(HTMLParser):
         if isFromRoot is True and root.id == _id:
             return root
 
+        getElementById = self.getElementById
         for child in root.children:
+
             if child.getAttribute('id') == _id:
                 return child
-            potential = self.getElementById(_id, child)
+
+            potential = getElementById(_id, child)
             if potential is not None:
                 return potential
+
         return None
 
     def getElementsByClassName(self, className, root='root'):
@@ -326,10 +344,14 @@ class AdvancedHTMLParser(HTMLParser):
         if isFromRoot is True and className in root.classNames:
             elements.append(root)
 
+        getElementsByClassName = self.getElementsByClassName
         for child in root.children:
+
             if className in child.classNames:
                 elements.append(child)
-            elements += self.getElementsByClassName(className, child)
+
+            elements += getElementsByClassName(className, child)
+
         return TagCollection(elements)
 
     def getElementsByAttr(self, attrName, attrValue, root='root'):
@@ -347,10 +369,14 @@ class AdvancedHTMLParser(HTMLParser):
         if isFromRoot is True and root.getAttribute(attrName) == attrValue:
             elements.append(root)
 
+        getElementsByAttr = self.getElementsByAttr
         for child in root.children:
+
             if child.getAttribute(attrName) == attrValue:
                 elements.append(child)
-            elements += self.getElementsByAttr(attrName, attrValue, child)
+
+            elements += getElementsByAttr(attrName, attrValue, child)
+
         return TagCollection(elements)
 
     def getElementsWithAttrValues(self, attrName, attrValues, root='root'):
@@ -365,6 +391,7 @@ class AdvancedHTMLParser(HTMLParser):
 
         '''
         (root, isFromRoot) = self._handleRootArg(root)
+
         if type(attrValues) != set:
             attrValues = set(attrValues)
         
@@ -386,10 +413,14 @@ class AdvancedHTMLParser(HTMLParser):
         if isFromRoot is True and filterFunc(root) is True:
             elements.append(root)
 
+        getElementsCustomFilter = self.getElementsCustomFilter
         for child in root.children:
+
             if filterFunc(child) is True:
                 elements.append(child)
-            elements += self.getElementsCustomFilter(filterFunc, child)
+
+            elements += getElementsCustomFilter(filterFunc, child)
+
         return TagCollection(elements)
 
 
@@ -410,10 +441,15 @@ class AdvancedHTMLParser(HTMLParser):
         if isFromRoot is True and filterFunc(root) is True:
             return root
 
+        getFirstElementCustomFilter = self.getFirstElementCustomFilter
+
         for child in root.children:
+
             if filterFunc(child) is True:
                 return child
-            subRet = self.getFirstElementCustomFilter(filterFunc, child)
+
+            subRet = getFirstElementCustomFilter(filterFunc, child)
+
             if subRet:
                 return subRet
 
@@ -628,7 +664,6 @@ class AdvancedHTMLParser(HTMLParser):
                 _values = tuple([x.lower() for x in _values])
 
             def _testFunc(em):
-                import pdb; pdb.set_trace()
                 text = em.text
                 if icontains:
                     text = text.lower()
@@ -730,6 +765,13 @@ class AdvancedHTMLParser(HTMLParser):
 #        return doctypeStr + ''.join([elem.outerHTML for elem in self.getRootNodes()])
 
 
+    # toHTML - Alias of getHTML
+    toHTML = getHTML
+
+    # asHTML - Alias of getHTML
+    asHTML = getHTML
+
+
     def getFormattedHTML(self, indent='  '):
         '''
             getFormattedHTML - Get formatted and xhtml of this document
@@ -753,7 +795,7 @@ class AdvancedHTMLParser(HTMLParser):
 
         self.root = None
         self.doctype = None
-        self.inTag = []
+        self._inTag = []
 
     def feed(self, contents):
         '''
@@ -781,6 +823,7 @@ class AdvancedHTMLParser(HTMLParser):
         else:
             with codecs.open(filename, 'r', encoding=self.encoding) as f:
                 contents = f.read()
+
         self.feed(contents)
 
     def parseStr(self, html):
@@ -790,6 +833,7 @@ class AdvancedHTMLParser(HTMLParser):
                 @param html <str> - valid HTML
         '''
         self.reset()
+
         if isinstance(html, bytes):
             self.feed(html.decode(self.encoding))
         else:
@@ -994,8 +1038,10 @@ class IndexedAdvancedHTMLParser(AdvancedHTMLParser):
 
     def _indexTagRecursive(self, tag):
         self._indexTag(tag)
+
+        _indexTagRecursive = self._indexTagRecursive
         for child in tag.children:
-            self._indexTagRecursive(child)
+            _indexTagRecursive(child)
 
     ######## Parsing #########
 
@@ -1099,11 +1145,14 @@ class IndexedAdvancedHTMLParser(AdvancedHTMLParser):
         if useIndex is True and self.indexTagNames is True:
             elements = self._tagNameMap.get(tagName, []) # Use .get here as to not create a lot of extra indexes on the defaultdict for misses
             if isFromRoot is False:
-                elements = [x for x in elements if self._hasTagInParentLine(x, root)]
+                _hasTagInParentLine = self._hasTagInParentLine
+                elements = [x for x in elements if _hasTagInParentLine(x, root)]
+
             return TagCollection(elements)
 
         return AdvancedHTMLParser.getElementsByTagName(self, tagName, root)
 
+
     def getElementsByName(self, name, root='root', useIndex=True):
         '''
             getElementsByName - Searches and returns all elements with a specific name.
@@ -1116,13 +1165,18 @@ class IndexedAdvancedHTMLParser(AdvancedHTMLParser):
 
         elements = []
         if useIndex is True and self.indexNames is True:
+
             elements = self._nameMap.get(name, [])
+
             if isFromRoot is False:
-                elements = [x for x in elements if self._hasTagInParentLine(x, root)]
+                _hasTagInParentLine = self._hasTagInParentLine
+                elements = [x for x in elements if _hasTagInParentLine(x, root)]
+
             return TagCollection(elements)
 
         return AdvancedHTMLParser.getElementsByName(self, name, root)
 
+
     def getElementById(self, _id, root='root', useIndex=True):
         '''
             getElementById - Searches and returns the first (should only be one) element with the given ID.
@@ -1134,8 +1188,11 @@ class IndexedAdvancedHTMLParser(AdvancedHTMLParser):
         (root, isFromRoot) = self._handleRootArg(root)
 
         if self.useIndex is True and self.indexIDs is True:
+
             element = self._idMap.get(_id, None)
+
             if isFromRoot is False and element is not None:
+
                 if self._hasTagInParentLine(element, root) is False:
                     element = None
                     
@@ -1156,9 +1213,12 @@ class IndexedAdvancedHTMLParser(AdvancedHTMLParser):
         (root, isFromRoot) = self._handleRootArg(root)
 
         if useIndex is True and self.indexClassNames is True:
+
             elements = self._classNameMap.get(className, [])
+
             if isFromRoot is False:
-                elements = [x for x in elements if self._hasTagInParentLine(x, root)]
+                _hasTagInParentLine = self._hasTagInParentLine
+                elements = [x for x in elements if _hasTagInParentLine(x, root)]
 
             return TagCollection(elements)
 
@@ -1178,9 +1238,13 @@ class IndexedAdvancedHTMLParser(AdvancedHTMLParser):
         (root, isFromRoot) = self._handleRootArg(root)
 
         if useIndex is True and attrName in self._otherAttributeIndexes:
+
             elements = self._otherAttributeIndexes[attrName].get(attrValue, [])
+
             if isFromRoot is False:
-                elements = [x for x in elements if self._hasTagInParentLine(x, root)]
+                _hasTagInParentLine = self._hasTagInParentLine
+                elements = [x for x in elements if _hasTagInParentLine(x, root)]
+
             return TagCollection(elements)
         
         return AdvancedHTMLParser.getElementsByAttr(self, attrName, attrValue, root)
@@ -1197,10 +1261,13 @@ class IndexedAdvancedHTMLParser(AdvancedHTMLParser):
         '''
         (root, isFromRoot) = self._handleRootArg(root)
 
-        if useIndex is True and attrName in self._otherAttributeIndexes:
+        _otherAttributeIndexes = self._otherAttributeIndexes
+        if useIndex is True and attrName in _otherAttributeIndexes:
+
             elements = TagCollection()
+
             for value in values:
-                elements += TagCollection(self._otherAttributeIndexes[attrName].get(value, []))
+                elements += TagCollection(_otherAttributeIndexes[attrName].get(value, []))
 
             return elements
 
