@@ -4,13 +4,14 @@
 
 from collections import OrderedDict
 
-import uuid
 import copy
+import re
+import uuid
 
 from .constants import PREFORMATTED_TAGS, IMPLICIT_SELF_CLOSING_TAGS, TAG_NAMES_TO_ADDITIONAL_ATTRIBUTES, COMMON_JAVASCRIPT_ATTRIBUTES, ALL_JAVASCRIPT_EVENT_ATTRIBUTES, TAG_ITEM_BINARY_ATTRIBUTES, TAG_ITEM_ATTRIBUTE_LINKS, TAG_ITEM_ATTRIBUTES_SPECIAL_VALUES, TAG_ITEM_CHANGE_NAME_FROM_ATTR, TAG_ITEM_CHANGE_NAME_FROM_ITEM
 from .SpecialAttributes import SpecialAttributesDict, StyleAttribute, AttributeNodeMap
 
-from .utils import escapeQuotes, tostr
+from .utils import escapeQuotes, tostr, stripWordsOnly
 
 __all__ = ('AdvancedTag', 'uniqueTags', 'TagCollection', 'FilterableTagCollection', 'toggleAttributesDOM', 'isTextNode', 'isTagNode')
 
@@ -72,7 +73,7 @@ def toggleAttributesDOM(isEnabled):
 
 # ADVANCED_TAG_RAW_ATTRIBUTES - These are tags which are just raw attributes on AdvancedTag
 #   Used to optimize access
-ADVANCED_TAG_RAW_ATTRIBUTES = set( ['tagName', '_attributes', 'text', 'blocks', 'classNames', 'isSelfClosing',
+ADVANCED_TAG_RAW_ATTRIBUTES = set( ['tagName', '_attributes', 'text', 'blocks', '_classNames', 'isSelfClosing',
                                     'children', 'parentNode', 'ownerDocument', 'uid', '_indent']
 )
 
@@ -113,12 +114,12 @@ class AdvancedTag(object):
         #         This could improve performance and memory usage both
         rawSet('text', '')
 
-        # TODO: Perhaps we should have helper methods like "textBlocks" and "tagBlocks" which perform a comprehension.
         rawSet('blocks', [''])
 
         # TODO: Maybe can refactor "children" into just being the "tagBlocks" from above?
         rawSet('children', [])
-        rawSet('classNames', [])
+
+        rawSet('_classNames', [])
         rawSet('isSelfClosing', isSelfClosing)
         rawSet('parentNode', None)
         rawSet('ownerDocument', ownerDocument)
@@ -132,10 +133,12 @@ class AdvancedTag(object):
 
         # If provided with a list of attributes as tuple(name, value)
         #   then apply those.
-        if attrList is not None:
+        if attrList:
+            myAttributes = self._attributes
+
             for key, value in attrList:
                 key = key.lower()
-                self._attributes[key] = value
+                myAttributes[key] = value
 
 
     def __setattr__(self, name, value):
@@ -162,6 +165,12 @@ class AdvancedTag(object):
         if name in ADVANCED_TAG_RAW_ATTRIBUTES:
             return object.__setattr__(self, name, value)
 
+        # Check for special "className"
+        if name == "className":
+            value = stripWordsOnly( tostr(value) )
+            object.__setattr__(self, '_classNames', [x for x in value.split(' ') if x])
+            return value
+
         # Check if this is one of the special items which map directly to attributes
         #    TAG_ITEM_ATTRIBUTE_LINKS - These attributes link directly to an html attribute, e.x. "id" or "name"
         #    TAG_NAMES_TO_ADDITIONAL_ATTRIBUTES - This is a map for specific dot-access attributes specific to
@@ -355,7 +364,7 @@ class AdvancedTag(object):
                 # Create a copy of the old text in this block for return
                 removedBlock = block[:]
                 # Remove first occurance of #text from matched block
-                self.blocks[i] = block.replace(text, '')
+                blocks[i] = block.replace(text, '')
                 break # remove should only remove FIRST occurace, per other methods
 
         # Regenerate the "text" property
@@ -395,7 +404,7 @@ class AdvancedTag(object):
                 removedBlocks.append( block[:] )
                 
                 # And replace the text within this matched block
-                self.blocks[i] = block.replace(text, '')
+                blocks[i] = block.replace(text, '')
 
         
         # Regenerate self.text
@@ -455,9 +464,11 @@ class AdvancedTag(object):
         child.parentNode = self
 
         # Associate owner document to child and all children recursive
-        child.ownerDocument = self.ownerDocument
+        ownerDocument = self.ownerDocument
+
+        child.ownerDocument = ownerDocument
         for subChild in child.getAllChildNodes():
-            subChild.ownerDocument = self.ownerDocument
+            subChild.ownerDocument = ownerDocument
 
         # Our tag cannot be self-closing if we have a child tag
         self.isSelfClosing = False
@@ -632,20 +643,23 @@ class AdvancedTag(object):
         # If #child is an AdvancedTag, we need to add it to both blocks and children.
         isChildTag = isTagNode(child)
 
+        myBlocks = self.blocks
+        myChildren = self.children
+
         # Find the index #beforeChild falls under current element
         try:
-            blocksIdx =  self.blocks.index(beforeChild)
+            blocksIdx =  myBlocks.index(beforeChild)
             if isChildTag:
-                childrenIdx = self.children.index(beforeChild)
+                childrenIdx = myChildren.index(beforeChild)
         except ValueError:
             # #beforeChild is not a child of this element. Raise error.
             raise ValueError('Provided "beforeChild" is not a child of element, cannot insert.')
         
         # Add to blocks in the right spot
-        self.blocks = self.blocks[:blocksIdx] + [child] + self.blocks[blocksIdx:]
+        self.blocks = myBlocks[:blocksIdx] + [child] + myBlocks[blocksIdx:]
         # Add to child in the right spot
         if isChildTag: 
-            self.children = self.children[:childrenIdx] + [child] + self.children[childrenIdx:]
+            self.children = myChildren[:childrenIdx] + [child] + myChildren[childrenIdx:]
         
         return child
 
@@ -667,18 +681,21 @@ class AdvancedTag(object):
 
         isChildTag = isTagNode(child)
 
+        myBlocks = self.blocks
+        myChildren = self.children
+
         # Determine where we need to insert this both in "blocks" and, if a tag, "children"
         try:
-            blocksIdx =  self.blocks.index(afterChild)
+            blocksIdx =  myBlocks.index(afterChild)
             if isChildTag:
-                childrenIdx = self.children.index(afterChild)
+                childrenIdx = myChildren.index(afterChild)
         except ValueError:
             raise ValueError('Provided "afterChild" is not a child of element, cannot insert.')
 
         # Append child to requested spot
-        self.blocks = self.blocks[:blocksIdx+1] + [child] + self.blocks[blocksIdx+1:]
+        self.blocks = myBlocks[:blocksIdx+1] + [child] + myBlocks[blocksIdx+1:]
         if isChildTag:
-            self.children = self.children[:childrenIdx+1] + [child] + self.children[childrenIdx+1:]
+            self.children = myChildren[:childrenIdx+1] + [child] + myChildren[childrenIdx+1:]
 
         return child
 
@@ -740,6 +757,18 @@ class AdvancedTag(object):
     #                Can be changed to "attributesDOM" for a less-useful but more-strict adherance to JS API
     attributes = attributesDict
 
+    @property
+    def attributesList(self):
+        '''
+            attributesList - Returns a copy of internal attributes as a list. Same as getAttributesList method.
+
+                @return list<tuple> - List of (key, value) tuples representing each attribute on this node
+                
+
+              @see getAttributesList
+              @see attributesDict
+        '''
+        return self.getAttributesList()
 
     @property
     def nextSibling(self):
@@ -751,19 +780,21 @@ class AdvancedTag(object):
                 @return <None/str/AdvancedTag> - None if there are no nodes (text or tag) in the parent after this node,
                                                     Otherwise the following node (text or tag)
         '''
+        parentNode = self.parentNode
+
         # If no parent, no siblings.
-        if not self.parentNode:
+        if not parentNode:
             return None
 
         # Determine index in blocks
-        myBlockIdx = self.parentNode.blocks.index(self)
+        myBlockIdx = parentNode.blocks.index(self)
 
         # If we are the last, no next sibling
-        if myBlockIdx == len(self.parentNode.blocks):
+        if myBlockIdx == len(parentNode.blocks) - 1:
             return None
 
         # Else, return the next block in parent
-        return self.parentNode.blocks[myBlockIdx+1]
+        return parentNode.blocks[ myBlockIdx + 1 ]
 
     @property
     def nextSiblingElement(self):
@@ -774,19 +805,21 @@ class AdvancedTag(object):
                 @return <None/AdvancedTag> - None if there are no children (tag) in the parent after this node,
                                                     Otherwise the following element (tag)
         '''
+        parentNode = self.parentNode
+
         # If no parent, no siblings
-        if not self.parentNode:
+        if not parentNode:
             return None
 
         # Determine the index in children
-        myElementIdx = self.parentNode.children.index(self)
+        myElementIdx = parentNode.children.index(self)
 
         # If we are last child, no next sibling
-        if myElementIdx == len(self.parentNode.children):
+        if myElementIdx == len(parentNode.children) - 1:
             return None
 
         # Else, return the next child in parent
-        return self.parentNode.children[myElementIdx+1]
+        return parentNode.children[myElementIdx+1]
         
     @property
     def previousSibling(self):
@@ -799,19 +832,21 @@ class AdvancedTag(object):
                 @return <None/str/AdvancedTag> - None if there are no nodes (text or tag) in the parent before this node,
                                                     Otherwise the previous node (text or tag)
         '''
+        parentNode = self.parentNode
+
         # If no parent, no previous sibling
-        if not self.parentNode:
+        if not parentNode:
             return None
 
         # Determine block index on parent of this node
-        myBlockIdx = self.parentNode.blocks.index(self)
+        myBlockIdx = parentNode.blocks.index(self)
         
         # If we are the first, no previous sibling
         if myBlockIdx == 0:
             return None
 
         # Else, return the previous block in parent
-        return self.parentNode.blocks[myBlockIdx-1]
+        return parentNode.blocks[myBlockIdx-1]
 
     @property
     def previousSiblingElement(self):
@@ -825,20 +860,76 @@ class AdvancedTag(object):
                                                     Otherwise the previous element (tag)
 
         '''
+        parentNode = self.parentNode
+
         # If no parent, no siblings
-        if not self.parentNode:
+        if not parentNode:
             return None
 
         # Determine this node's index in the children of parent
-        myElementIdx = self.parentNode.children.index(self)
+        myElementIdx = parentNode.children.index(self)
         
         # If we are the first child, no previous element
         if myElementIdx == 0:
             return None
 
         # Else, return previous element tag
-        return self.parentNode.children[myElementIdx-1]
-        
+        return parentNode.children[myElementIdx-1]
+
+
+    @property
+    def tagBlocks(self):
+        '''
+            tagBlocks - Property. 
+                        Returns all the blocks which are direct children of this node, where that block is a tag (not text)
+
+                NOTE: This is similar to .children , and you should probably use .children instead except within this class itself
+
+                @return list<AdvancedTag> - A list of direct children which are tags.
+        '''
+        myBlocks = self.blocks
+
+        return [block for block in myBlocks if issubclass(block.__class__, AdvancedTag)]
+
+
+    def getBlocksTags(self):
+        '''
+            getBlocksTags - Returns a list of tuples referencing the blocks which are direct children of this node, and the block is an AdvancedTag.
+
+                The tuples are ( block, blockIdx ) where "blockIdx" is the index of self.blocks wherein the tag resides.
+
+                @return list< tuple(block, blockIdx) > - A list of tuples of child blocks which are tags and their index in the self.blocks list
+        '''
+        myBlocks = self.blocks
+
+        return [ (myBlocks[i], i) for i in range( len(myBlocks) ) if issubclass(myBlocks[i].__class__, AdvancedTag) ]
+
+
+    @property
+    def textBlocks(self):
+        '''
+            textBlocks - Property. 
+                        Returns all the blocks which are direct children of this node, where that block is a text (not a tag)
+
+                @return list<AdvancedTag> - A list of direct children which are text.
+        '''
+        myBlocks = self.blocks
+
+        return [block for block in myBlocks if not issubclass(block.__class__, AdvancedTag)]
+
+
+    def getBlocksText(self):
+        '''
+            getBlocksText - Returns a list of tuples referencing the blocks which are direct children of this node, and the block is a text node (not an AdvancedTag)
+
+                The tuples are ( block, blockIdx ) where "blockIdx" is the index of self.blocks wherein the text resides.
+
+                @return list< tuple(block, blockIdx) > - A list of tuples of child blocks which are not tags and their index in the self.blocks list
+        '''
+        myBlocks = self.blocks
+
+        return [ (myBlocks[i], i) for i in range( len(myBlocks) ) if not issubclass(myBlocks[i].__class__, AdvancedTag) ]
+
 
     def getChildren(self):
         '''
@@ -978,12 +1069,15 @@ class AdvancedTag(object):
 
             @return - TagCollection of elements
         '''
+        parentNode = self.parentNode
         # If no parent, no peers
-        if not self.parentNode:
+        if not parentNode:
             return None
 
+        peers = parentNode.children
+
         # Otherwise, get all children of parent excluding this node
-        return TagCollection([peer for peer in self.parentNode.children if peer is not self])
+        return TagCollection([peer for peer in peers if peer is not self])
 
     @property
     def peers(self):
@@ -1048,14 +1142,23 @@ class AdvancedTag(object):
         '''
         return self.parentNode
 
+    @property
+    def className(self):
+        '''
+            className - property, string of 'class' attribute
+
+            @return <str> - Class attribute, or empty string if not set
+        '''
+        return ' '.join(self.classList)
+
     @property
     def classList(self):
         '''
-            classList - get a list of the class names ( the "class" attribute ) for this element
+            classList - get a copy of the list of the class names ( the "class" attribute ) for this element
 
                 @return list<str> - A list of the class names for this element
         '''
-        return self.classNames
+        return self._classNames[:]
 
     def getUid(self):
         '''
@@ -1125,12 +1228,14 @@ class AdvancedTag(object):
         if self.isSelfClosing is True:
             return ''
 
+        tagName = self.tagName
+
         # Do not add any indentation to the end of preformatted tags.
-        if self._indent and self.tagName in PREFORMATTED_TAGS:
-            return "</%s>" %(self.tagName)
+        if self._indent and tagName in PREFORMATTED_TAGS:
+            return "</%s>" %(tagName, )
 
         # Otherwise, indent the end of this tag
-        return "%s</%s>" %(self._indent, self.tagName)
+        return "%s</%s>" %(self._indent, tagName)
 
     @property
     def innerHTML(self):
@@ -1260,26 +1365,39 @@ class AdvancedTag(object):
 
             @return <bool> - True if provided class is present, otherwise False
         '''
-        return bool(className in self.classNames)
-     
+        return bool(className in self._classNames)
+
+
     def addClass(self, className):
         '''
             addClass - append a class name to the end of the "class" attribute, if not present
 
                 @param className <str> - The name of the class to add
         '''
+        className = stripWordsOnly(className)
+
+        if not className:
+            return None
+
+        if ' ' in className:
+            # Multiple class names passed, do one at a time
+            for oneClassName in className.split(' '):
+                self.addClass(oneClassName)
+            return
+
+        myClassNames = self._classNames
+
         # Do not allow duplicates
-        if className in self.classNames:
+        if className in myClassNames:
             return
 
         # Regenerate "classNames" and "class" attr.
         #   TODO: Maybe those should be properties?
-        self.classNames.append(className)
-        self.className = ' '.join(self.classNames)
-        self._attributes._direct_set('class', self.className)
+        myClassNames.append(className)
 
         return None
 
+
     def removeClass(self, className):
         '''
             removeClass - remove a class name if present. Returns the class name if  removed, otherwise None.
@@ -1288,15 +1406,26 @@ class AdvancedTag(object):
 
                 @return <str> - The class name removed if one was removed, otherwise None if #className wasn't present
         '''
-        # If not present, this is a no-op
-        if className not in self.classNames:
+        className = stripWordsOnly(className)
+
+        if not className:
             return None
 
-        self.classNames.remove(className)
+        if ' ' in className:
+            # Multiple class names passed, do one at a time
+            for oneClassName in className.split(' '):
+                self.removeClass(oneClassName)
+            return
+
+        myClassNames = self._classNames
+
+        # If not present, this is a no-op
+        if className not in myClassNames:
+            return None
+
+
+        myClassNames.remove(className)
 
-        # Regenerate "classNames" and "class" attr.
-        self.className = ' '.join(self.classNames)
-        self._attributes._direct_set('class', self.className)
         return className
 
 
@@ -1350,10 +1479,12 @@ class AdvancedTag(object):
 
             @return - String of current value of "style" after change is made.
         '''
-        if 'style' not in self._attributes:
-            self._attributes['style'] = "%s: %s" %(styleName, styleValue)
+        myAttributes = self._attributes
+
+        if 'style' not in myAttributes:
+            myAttributes['style'] = "%s: %s" %(styleName, styleValue)
         else:
-            setattr(self._attributes['style'], styleName, styleValue)
+            setattr(myAttributes['style'], styleName, styleValue)
 #        setattr(self.style, styleName, styleValue)
 
     def setStyles(self, styleUpdatesDict):
@@ -1368,18 +1499,60 @@ class AdvancedTag(object):
 
             @return - String of current value of "style" after change is made.
         '''
+        setStyleMethod = self.setStyle
         for newName, newValue in styleUpdatesDict.items():
-            self.setStyle(newName, newValue)
+            setStyleMethod(newName, newValue)
 
         return self.style
 
 
-    def __str__(self):
+    def _old__str__(self):
         '''
-            __str__ - Returns start tag, inner text, and end tag
+            _old__str__ - The old impl for __str__ (before 7.3.1) which just contains the start tag, inner text, and end tag.
+
+                I think this can cause some unexpected results, especially with no "toHTML" methods etc.
+
+                @return <str> - Start tag, inner text, and end tag
         '''
         return self.getStartTag() + self.text + self.getEndTag()
 
+
+    def __str__(self):
+        '''
+            __str__ - Returns the HTML representation for this tag (including children).
+
+                NOTE: This changed in 7.3.1 to be equivilant to self.outerHTML (or to new getHTML method, which is the same).
+
+                The old method just included the start tag, the joined direct text node children, and the end tag.
+                    This compacts well for debug display, but doesn't give a clear picture of what's going on.
+
+                The old method is still available as AdvancedTag._old__str__
+
+                To revert str(myTag) back to the hold behaviour:
+
+                    from AdvancedHTMLParser.Tags import AdvancedTag
+
+                    AdvancedTag.__str__ = AdvancedTag._old__str__
+        '''
+        return self.outerHTML
+
+
+    def toHTML(self):
+        '''
+            toHTML - Get the HTML representation of this tag and all children
+
+              @return <str> - HTML with this tag as the root
+        '''
+        return self.outerHTML
+
+
+    # asHTML - Alias of toHTML
+    asHTML = toHTML
+
+    # getHTML - Alias of toHTML
+    getHTML = toHTML
+
+
     def __getitem__(self, key):
         return self.children[key]
 
@@ -1592,8 +1765,11 @@ class AdvancedTag(object):
             if self.tagName != other.tagName:
                 return False
 
-            attributeKeysSelf = list(self._attributes.keys())
-            attributeKeysOther = list(other._attributes.keys())
+            myAttributes = self._attributes
+            otherAttributes = other._attributes
+
+            attributeKeysSelf = list(myAttributes.keys())
+            attributeKeysOther = list(otherAttributes.keys())
         except:
             return False
 
@@ -1603,7 +1779,7 @@ class AdvancedTag(object):
 
         for key in attributeKeysSelf:
 
-            if self._attributes.get(key) != other._attributes.get(key):
+            if myAttributes.get(key) != otherAttributes.get(key):
                 return False
 
         return True
@@ -1760,15 +1936,19 @@ class TagCollection(list):
 
     def __add__(self, others):
         # Maybe this can be optimized by changing self.uids to a dictionary, and using appending the set difference
+        hasTag = self._hasTag
+
         ret = TagCollection(self[:])
         for other in others:
-            if self._hasTag(other) is False:
+            if hasTag(other) is False:
                 ret.append(other)
         return ret
 
     def __iadd__(self, others):
+        hasTag = self._hasTag
+
         for other in others:
-            if self._hasTag(other) is False:
+            if hasTag(other) is False:
                 self.append(other)
         return self
 
@@ -1776,16 +1956,20 @@ class TagCollection(list):
 
 
     def __sub__(self, others):
+        hasTag = self._hasTag
+
         ret = TagCollection(self[:])
 
         for other in others:
-            if self._hasTag(other) is True:
+            if hasTag(other) is True:
                 ret.remove(other)
         return ret
 
     def __isub__(self, others):
+        hasTag = self._hasTag
+
         for other in others:
-            if self._hasTag(other) is True:
+            if hasTag(other) is True:
                 self.remove(other)
         return self
             
